clear
clc

// Parâmetros do Sistema
m = 25222;
g = 9.8;
l = 22;
Jo = 4.07*10^6;

// Parâmetros Numéricos
tmax = 20;
t = 0:0.01:tmax;

// Variáveis de Estado/Entrada
theta = 0;
v_ar = 17.1;

function ut = u(t)
    Fv = 0;
    if t < 1 then
        Fps = 0;
    elseif t < 1.5
        Fps = 100888;
    else
        Fps = 0;
    end
    Fp = 268000;
    phi = 0;
    ut = [Fv,Fps,Fp,phi];
endfunction

// Modelo Não-Linear
x0e = [0;-2;250;-20;0;0];

function dx = foguetenl(t,x,u)
    U = u(t);
    dx(1) = x(2);
    dx(2) = (U(1) + U(2) * cos(x(5)) + U(3) * sin(x(5) + U(4))) / m;
    dx(3) = x(4);
    dx(4) = (U(3) * cos(x(5) + U(4)) - U(2) * sin(x(5)) - m*g) / m;
    dx(5) = x(6);
    dx(6) = (U(2) - U(3) * sin(U(4))) * l/Jo;
endfunction

Xnl = ode(x0e,0,t,list(foguetenl,u));

// Modelo Linear
function xn = nomTraj(t)
    if t < 1.1 then
        xn(1) = -2*t;
        xn(2) = -2;
    else
        xn(1) = -2;
        xn(2) = 0;
    end
    xn(3) = 250 - 20 * t + (268000/m-g) *t^2/2;
    xn(4) = - 20 + (268000/m-g) * t;
    xn(5) = 0;
    xn(6) = 0;
endfunction

function du = Du(t)
    du(1) = 0;
    du(2) = 0;
    du(3) = 0;
    du(4) = 0;
endfunction

dx = [0;0;0;0;0;0];

for i = 1:length(t)
    du = Du(t(i)); // Desvio das entradas
    un = u(t(i)) - du'; // Entradas nominais (Real - Desvio)
    xn = nomTraj(t(i)); // Trajetória Nominal

    A = [0 1 0 0 0 0;
         0 0 0 0 (-un(2)*sin(xn(5))+un(3)*cos(xn(5)+un(4)))/m 0;
         0 0 0 1 0 0;
         0 0 0 0 -(un(3)*sin(xn(5)+un(4))+un(2)*cos(xn(5)))/m 0;
         0 0 0 0 0 1;
         0 0 0 0 0 0];
    
    B = [0 0 0 0;
         1/m cos(xn(5))/m sin(xn(5)+un(4))/m un(3)*cos(xn(5)+un(4))/m;
         0 0 0 0;
         0 -sin(xn(5))/m cos(xn(5)+un(4))/m -un(3)*sin(xn(5)+un(4))/m;
         0 0 0 0;
         0 l/Jo -sin(un(4))*l/Jo -un(3)*cos(un(4))*l/Jo];
    
    C = [1 0 0 0 0 0;
         0 1 0 0 0 0;
         0 0 1 0 0 0;
         0 0 0 1 0 0;
         0 0 0 0 1 0];
    
    D = [0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0];
     
     dy = C * dx + D * du;
     yn = C * xn + D * un';
     Xl(:,i) = yn + dy;
     
     dx = dx + A * dx + B * du;
end

for i = 1:length(t)
    k(i,:) = u(t(i));
end

// Plots
xset('window',1)
xtitle('Deslocamento no Plano','X [m]','Y [m]')
plot2d(Xl(1,:),Xl(3,:),2)
plot2d(Xnl(1,:),Xnl(3,:),3)
plot2d(linspace(-300,300,length(t)),ones(t)*l,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',2)
xtitle('Deslocamento vertical','Tempo [s]','Y [m]')
plot2d(t,Xl(3,:),2)
plot2d(t,Xnl(3,:),3)
plot2d(t,ones(t)*l,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',3)
xtitle('Velocidade vertical','Tempo [s]','Vy [m/s]')
plot2d(t,Xl(4,:),2)
plot2d(t,Xnl(4,:),3)
plot2d(t,-ones(t)*6,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',4)
xtitle('Velocidade horizontal','Tempo [s]','Vx [m/s]')
plot2d(t,Xl(2,:),2)
plot2d(t,Xnl(2,:),3)
plot2d(t,ones(t)*5.4,1)
plot2d(t,-ones(t)*5.4,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',5)
xtitle('Ângulo de arfagem','Tempo [s]','Ângulo [rad]')
plot2d(t,Xl(5,:),2)
plot2d(t,Xnl(5,:),3)
plot2d(t,ones(t)*0.4014,1)
plot2d(t,-ones(t)*0.4014,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])clear
clc

// Parâmetros do Sistema
m = 25222;
g = 9.8;
l = 22;
Jo = 4.07*10^6;

// Parâmetros Numéricos
tmax = 20;
t = 0:0.01:tmax;

// Variáveis de Estado/Entrada
theta = 0;
v_ar = 17.1;

function ut = u(t)
    Fv = 0;
    if t < 1 then
        Fps = 0;
    elseif t < 1.5
        Fps = 100888;
    else
        Fps = 0;
    end
    Fp = 268000;
    phi = 0;
    ut = [Fv,Fps,Fp,phi];
endfunction

// Modelo Não-Linear
x0e = [0;-2;250;-20;0;0];

function dx = foguetenl(t,x,u)
    U = u(t);
    dx(1) = x(2);
    dx(2) = (U(1) + U(2) * cos(x(5)) + U(3) * sin(x(5) + U(4))) / m;
    dx(3) = x(4);
    dx(4) = (U(3) * cos(x(5) + U(4)) - U(2) * sin(x(5)) - m*g) / m;
    dx(5) = x(6);
    dx(6) = (U(2) - U(3) * sin(U(4))) * l/Jo;
endfunction

Xnl = ode(x0e,0,t,list(foguetenl,u));

// Modelo Linear
function xn = nomTraj(t)
    if t < 1.1 then
        xn(1) = -2*t;
        xn(2) = -2;
    else
        xn(1) = -2;
        xn(2) = 0;
    end
    xn(3) = 250 - 20 * t + (268000/m-g) *t^2/2;
    xn(4) = - 20 + (268000/m-g) * t;
    xn(5) = 0;
    xn(6) = 0;
endfunction

function du = Du(t)
    du(1) = 0;
    du(2) = 0;
    du(3) = 0;
    du(4) = 0;
endfunction

dx = [0;0;0;0;0;0];

for i = 1:length(t)
    du = Du(t(i)); // Desvio das entradas
    un = u(t(i)) - du'; // Entradas nominais (Real - Desvio)
    xn = nomTraj(t(i)); // Trajetória Nominal

    A = [0 1 0 0 0 0;
         0 0 0 0 (-un(2)*sin(xn(5))+un(3)*cos(xn(5)+un(4)))/m 0;
         0 0 0 1 0 0;
         0 0 0 0 -(un(3)*sin(xn(5)+un(4))+un(2)*cos(xn(5)))/m 0;
         0 0 0 0 0 1;
         0 0 0 0 0 0];
    
    B = [0 0 0 0;
         1/m cos(xn(5))/m sin(xn(5)+un(4))/m un(3)*cos(xn(5)+un(4))/m;
         0 0 0 0;
         0 -sin(xn(5))/m cos(xn(5)+un(4))/m -un(3)*sin(xn(5)+un(4))/m;
         0 0 0 0;
         0 l/Jo -sin(un(4))*l/Jo -un(3)*cos(un(4))*l/Jo];
    
    C = [1 0 0 0 0 0;
         0 1 0 0 0 0;
         0 0 1 0 0 0;
         0 0 0 1 0 0;
         0 0 0 0 1 0];
    
    D = [0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0];
     
     dy = C * dx + D * du;
     yn = C * xn + D * un';
     Xl(:,i) = yn + dy;
     
     dx = dx + A * dx + B * du;
end

for i = 1:length(t)
    k(i,:) = u(t(i));
end

// Plots
xset('window',1)
xtitle('Deslocamento no Plano','X [m]','Y [m]')
plot2d(Xl(1,:),Xl(3,:),2)
plot2d(Xnl(1,:),Xnl(3,:),3)
plot2d(linspace(-300,300,length(t)),ones(t)*l,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',2)
xtitle('Deslocamento vertical','Tempo [s]','Y [m]')
plot2d(t,Xl(3,:),2)
plot2d(t,Xnl(3,:),3)
plot2d(t,ones(t)*l,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',3)
xtitle('Velocidade vertical','Tempo [s]','Vy [m/s]')
plot2d(t,Xl(4,:),2)
plot2d(t,Xnl(4,:),3)
plot2d(t,-ones(t)*6,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',4)
xtitle('Velocidade horizontal','Tempo [s]','Vx [m/s]')
plot2d(t,Xl(2,:),2)
plot2d(t,Xnl(2,:),3)
plot2d(t,ones(t)*5.4,1)
plot2d(t,-ones(t)*5.4,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])

xset('window',5)
xtitle('Ângulo de arfagem','Tempo [s]','Ângulo [rad]')
plot2d(t,Xl(5,:),2)
plot2d(t,Xnl(5,:),3)
plot2d(t,ones(t)*0.4014,1)
plot2d(t,-ones(t)*0.4014,1)
xgrid(1)
hl = legend(['Linear';'Não Linear';'Limite'])
